/*mmSAP 1.3 by Michael Kalouš (BAKTRA Software)*/

#include "mmsap_config.h"

/*Badly configured = error*/
#ifndef MMSAP_SOUND_SYSTEM_ALSA
#ifndef MMSAP_SOUND_SYSTEM_SDL
#error "Not configured for SDL or for ALSA !"
#endif
#endif



/*Function Prototypes*/
void setupPrefsDialog();

/*SAP Library headers*/
#include "libsap/sapLib.h"

/*GTKmm Library headers*/
#include <gtkmm.h>

/*Preferences*/
#include "preferences.h"
#include "preferences.cpp"
Preferences prefs;

#ifdef MMSAP_SOUND_SYSTEM_ALSA
#include "sapaudio-als.h"
#endif

#ifdef MMSAP_SOUND_SYSTEM_SDL
#include "sapaudio-sdl.h"
#endif

/*Global variables*/
short int buffer[4096*8];	/*Original 32768*/
long int zerorun=0;			/*Silence detection*/
SAPAudio* audioPtr;			/*To acess audio object*/
Gtk::Label* labelPtr;		/*To acess subsong label*/
short int quirk[8];			/*Fake buffer*/
short int zraccel;		/*Silence detection acceleration suplementery variable*/
sapMUSICstrc* sapHeader;
char argbuf[256];

/*Include appropriate implementation of SAPAudio class*/
#ifdef MMSAP_SOUND_SYSTEM_ALSA
#include "sapaudio-als.c"
#endif

#ifdef MMSAP_SOUND_SYSTEM_SDL
#include "sapaudio-sdl.c"
#endif

SAPAudio audio;

/* GUI made with Glade*******************************************************/

#include <gtkmm/filechooserdialog.h>

/*File chooser dialog*/
Gtk::FileChooserDialog* fcptr;
Gtk::Button* btOpenptr;
Gtk::Button* btCancelptr;


/*About dialog*/
#include "ui/dlgAbout_glade.hh"
#include "ui/dlgAbout_glade.cc"
#include "ui/dlgAbout.hh"
#include "ui/dlgAbout.cc"


dlgAbout* aboutPtr;

/*Preferences dialog*/
#include "ui/dlgPreferences_glade.hh"
#include "ui/dlgPreferences_glade.cc"
#include "ui/dlgPreferences.hh"
#include "ui/dlgPreferences.cc"

dlgPreferences* preferencesPtr;


/*Main window*/
#include "ui/winMainWindow_glade.hh"
#include "ui/winMainWindow_glade.cc"
#include "ui/winMainWindow.hh"
#include "ui/winMainWindow.cc"


/********************************************************************************************/
void setupPrefsDialog() {
	if (prefs.useDefaultDir==0) {
		preferencesPtr->chbUseDefaultDirectory->set_active(false);
	}
	else {
		preferencesPtr->chbUseDefaultDirectory->set_active(true);
	}

	if (prefs.ignoreDefaultSong==0) {
		preferencesPtr->chbIgnoreDefaultSong->set_active(false);
	}
	else {
		preferencesPtr->chbIgnoreDefaultSong->set_active(true);
	}

	if (prefs.expandInfo==0) {
		preferencesPtr->chbExpandInfo->set_active(false);
	}
	else {
		preferencesPtr->chbExpandInfo->set_active(true);
	}
	
	preferencesPtr->entryDefaultDirectory->set_text(prefs.defaultdir);
	preferencesPtr->hscSilence->set_value(prefs.silenceSeconds);
}

/********************************************************************************************/
int main(int argc,char* argv[]) {
	
	audioPtr=&audio;
	
	Gtk::Main kit(argc, argv);
	
	/*Window and dialog initialization*/
	winMainWindow pWindow;
	dlgAbout aboutDialog;
	aboutPtr=&aboutDialog;
	dlgPreferences preferencesDialog;
	preferencesPtr=&preferencesDialog;

	/*Version*/
	aboutDialog.lblVersion->set_text(MMSAP_VERSION);	

	/*Load preferences*/
	prefs.load();
	prefs.silenceSamples=prefs.silenceSeconds*44100;
	/*Setup the preferences dialog*/
	setupPrefsDialog();
	

	
	Gtk::FileChooserDialog fileChooser(pWindow,"Open SAP",Gtk::FILE_CHOOSER_ACTION_OPEN);
	fcptr=&fileChooser;
	btOpenptr = fileChooser.add_button("OPEN",Gtk::RESPONSE_OK);
	btCancelptr = fileChooser.add_button("Cancel",Gtk::RESPONSE_CANCEL);
		
	fileChooser.set_transient_for(pWindow);
	aboutDialog.set_transient_for(pWindow);

	/*Update the file chooser if necessary*/
	if (prefs.useDefaultDir==1) {
		fileChooser.set_current_folder(prefs.defaultdir);
	}
	
	/*Expand the expander when this option is set-up*/
	if (prefs.expandInfo==1) pWindow.expander1->set_expanded(true);
	
	pWindow.show();
	
	pWindow.set_resizable(false);
	labelPtr=pWindow.lblNumbers;
	
	
	audio.initAudio();
	if (argc==2) {
		int r=0;
		if (strlen(argv[1])<255) {
			strcpy(argbuf,argv[1]);
			int i;
			for (i=strlen(argbuf)-1;i>=0;i--) {
				if (argbuf[i]=='\n'|| argbuf[i]=='\r') {argbuf[i]=0; break;}
			}
			r=audio.openSAP(argbuf);
			if (r<0) {
				pWindow.lblTitle->set_label("ERROR: Unable to open SAP tune");
				pWindow.expander1->set_expanded(true);
				printf("Attempt to open: #%s#, %d\n",argbuf,strlen(argv[1]));
			}
			else {
				audio.playSAP();
				if (sapHeader->commentBuffer!=NULL) {
					pWindow.lblTitle->set_label(sapHeader->commentBuffer);
				}
				audio.updateSongChr();
				pWindow.lblNumbers->set_text(audio.songChr);
			
			}
		}
	}

	if (argc!=1 && argc!=2) {
		puts("Invalid parameters. Usage: mmsap [filespec]");
		exit(1);
	}
	
	Gtk::Main::run(pWindow);

	/*Cleanup*/
	audio.closeSAP();
	audio.destroyAudio();
	
	/*Save preferences*/
	prefs.save();	

}
