pthread_t playbackThread;


/* Audio thread routine *******************************************************/
void *audio_thread_run(void* data) {

	while(audioPtr->anyOpened) {
		while(audioPtr->getPaused()==0) {

			/*Fill the buffer*/
			sapRenderBuffer(buffer,4096);
			
			/*When SAP tune is closed, exit this loop*/
			if (!audioPtr->anyOpened) break;		
			
			/*Detect silence*/
			for(int i=0;i<4096;i++) {
				zraccel=buffer[i];
				if (zraccel!=-1 && zraccel!=0 ) {
				zerorun=0;
				break;
				}
			}
			zerorun+=4096;
						
			while ((snd_pcm_writei(audioPtr->playback_handle, buffer, 4096)) < 0) {
        		snd_pcm_prepare(audioPtr->playback_handle);
        	}
			
			/*Zerorun test*/
			if (zerorun>prefs.silenceSamples) {
				zerorun=0;
				audioPtr->nextSubsong();
				if (audioPtr->oneRepeat) audioPtr->previousSubsong();
				audioPtr->updateSongChr();
				labelPtr->set_text(audioPtr->songChr);
				puts("zerorun here");
			}
		}
		/*TODO In this place the audio is stopped or paused. The thread should sleep
		a little and not overload the CPU*/
		
	}
	/*End of thread*/
	puts("thread finished");
	
}




/* SAPAudio implementation for ALSA *********************************************/

/*Implementation*/
void SAPAudio::initAudio() {
	
	/*Setup the initial values*/
	sampleRate=44100;
	anyOpened=0;
	isPlaying=0;
	currentSong=0;
	maximumSong=0;
	paused=1;
	oneRepeat=0;
	
	/*ALSA*/
	int err;
    stream = SND_PCM_STREAM_PLAYBACK;	//Playback stream
	snd_pcm_hw_params_t *hw_params; //Hardware parameters
	
	/*Hardware param allocation and setup*/
	if ((err = snd_pcm_open (&playback_handle, "hw:0,0", SND_PCM_STREAM_PLAYBACK, 0)) < 0) {
		puts("ALSA ERROR: Playback handle");
    }
 
    if ((err = snd_pcm_hw_params_malloc (&hw_params)) < 0) {
		puts("ALSA ERROR: Hw params alloc");
    }

    if ((err = snd_pcm_hw_params_any (playback_handle, hw_params)) < 0) {
		puts("ALSA ERROR: Hw params any");
    }

    if ((err = snd_pcm_hw_params_set_access (playback_handle, hw_params, SND_PCM_ACCESS_RW_INTERLEAVED)) < 0) {
		puts("ALSA ERROR: Hw params set acess");
    }

    if ((err = snd_pcm_hw_params_set_format (playback_handle, hw_params, SND_PCM_FORMAT_S16_LE)) < 0) {
		puts("ALSA ERROR: Hw params set format");
    }

    if ((err = snd_pcm_hw_params_set_rate_near (playback_handle, hw_params, &sampleRate, 0)) < 0) {
		puts("ALSA ERROR: Hw params set rate near");
    }

    if ((err = snd_pcm_hw_params_set_channels (playback_handle, hw_params, 2)) < 0) {
		puts("ALSA ERROR: Hw params set channels");
    }

    if ((err = snd_pcm_hw_params (playback_handle, hw_params)) < 0) {
		puts("ALSA ERROR: snd_pcm_hw_params");

    }

	/*Parameters are setup, let's free them*/
    snd_pcm_hw_params_free (hw_params);
	
}

void SAPAudio::destroyAudio() {
	snd_pcm_close(playback_handle);
}

int SAPAudio::openSAP(char* filespec) {
	if (anyOpened) closeSAP();
	
	
	sapHeader = sapLoadMusicFile(filespec);
	
	if (sapHeader==NULL) {
		puts("ERROR: Unable to load SAP tune");
		anyOpened=0;
		return -1;
	}
	
	stopped=1;
	paused=1;
	
	anyOpened=1;
	/*Setup the current song and the maximum song*/
	currentSong=sapHeader->defSong;
	maximumSong=sapHeader->numOfSongs;
	
	if (maximumSong<1) maximumSong=1;
	
	if (currentSong<0 || currentSong>=maximumSong) currentSong=0;

	/*Default song can be ignored*/
	if (prefs.ignoreDefaultSong==1) currentSong=0;
	
	sapPlaySong(currentSong);
    sapRenderBuffer(quirk,2);

	/*Start the thread*/
	pthread_create(&playbackThread, NULL, audio_thread_run, NULL);

	return 0;
	
}

void SAPAudio::closeSAP() {
	if (!anyOpened) return;
	if (stopped==0) stopSAP();
	anyOpened=0;

	/*Join the playback thread. Let it exit it's code*/
	pthread_join (playbackThread, NULL);
}

void SAPAudio::stopSAP() {
	if(!anyOpened) return;
	paused=1;
	stopped=1;
	
	
}

void SAPAudio::playSAP() {
	if (!anyOpened) return;
	if (stopped==0) {
		paused=!paused;
	}
	else {
		stopped=0;
		paused=0;
		sapPlaySong(currentSong);
		sapRenderBuffer(quirk,2);
		sapPlaySong(currentSong);
		
	}
	
}

void SAPAudio::nextSubsong() {
	
	if(!anyOpened) return;
	currentSong=(currentSong+1) % maximumSong;
	sapPlaySong(currentSong);
	
}

void SAPAudio::previousSubsong() {
	if(!anyOpened) return;
	if (currentSong>0) currentSong--; else currentSong=maximumSong-1;
	sapPlaySong(currentSong);
	
}

void SAPAudio::updateSongChr() {
	if (anyOpened) {
		sprintf(songChr,"%d/%d",currentSong+1,maximumSong);
	}
	
}

int SAPAudio::getPaused() {
	return paused;
}

 
