void bufferFiller(void *userdata, Uint8 *stream, int len) {
	
	sapRenderBuffer(buffer,len/4);
	memcpy(stream,buffer,len);
	
	for(int i=0;i<len/4;i++) {
		zraccel=buffer[i];
		if (zraccel!=-1 && zraccel!=0 ) {
			zerorun=0;
			break;
		}
	}
	zerorun+=len/4;
	
	if (zerorun>prefs.silenceSamples) {
		zerorun=0;
		audioPtr->nextSubsong();
		audioPtr->updateSongChr();
		labelPtr->set_text(audioPtr->songChr);
	}
	
}

SDL_AudioSpec obtainedSpec;

/*Implementation*/
void SAPAudio::initAudio() {
	/*Initialize SDL*/
	int i;
	i=SDL_Init(SDL_INIT_AUDIO);
	
	/*If failed*/
	if (i<0) {
		puts("ERROR: Unable to initialize SDL audio system");
		puts(SDL_GetError());
		exit(-1);
	}
	
	/*Setup the initial values*/
	sampleRate=44100;
	anyOpened=0;
	isPlaying=0;
	currentSong=0;
	maximumSong=0;
	paused=1;
	
}

void SAPAudio::destroyAudio() {
	SDL_Quit();
}

int SAPAudio::openSAP(char* filespec) {
	if (anyOpened) closeSAP();
	
	
	sapHeader = sapLoadMusicFile(filespec);
	if (sapHeader==NULL) {
		puts("ERROR: Unable to load SAP tune");
		anyOpened=0;
		return -1;
	}
	
	stopped=1;
	paused=1;
	
	SDL_AudioSpec desiredSpec;
	desiredSpec.freq=sampleRate;
	desiredSpec.format=AUDIO_S16;
	desiredSpec.channels=0;	
	desiredSpec.samples=4096;//buffer size, original 4096
	desiredSpec.callback=bufferFiller;
	desiredSpec.userdata=this;
	
	int i;
	i=SDL_OpenAudio(&desiredSpec,&obtainedSpec);
	
	/*If failed*/
	if (i<0) {
		puts("ERROR: Unable to perform OpenAudio");
		puts(SDL_GetError());
		return -1;
	}
	
	anyOpened=1;
	/*Setup the current song and the maximum song*/
	currentSong=sapHeader->defSong;
	maximumSong=sapHeader->numOfSongs;
	
	if (maximumSong<1) maximumSong=1;
	
	if (currentSong<0 || currentSong>=maximumSong) currentSong=0;
	sapPlaySong(currentSong);
	sapRenderBuffer(quirk,2);
	//sapPlaySong(currentSong);
	return 0;
	
}

void SAPAudio::closeSAP() {
	if (!anyOpened) return;
	if (stopped==0) stopSAP();
	SDL_CloseAudio();
	anyOpened=0;

}

void SAPAudio::stopSAP() {
	if(!anyOpened) return;
	paused=1;
	stopped=1;
	SDL_PauseAudio(paused);
	
	
}

void SAPAudio::playSAP() {
	if (!anyOpened) return;
	if (stopped==0) {
		paused=!paused;
	}
	else {
		stopped=0;
		paused=0;
		sapPlaySong(currentSong);
		sapRenderBuffer(quirk,2);
		sapPlaySong(currentSong);
		
	}
	SDL_PauseAudio(paused);
}

void SAPAudio::nextSubsong() {
	
	if(!anyOpened) return;
	currentSong=(currentSong+1) % maximumSong;
	sapPlaySong(currentSong);
	
}

void SAPAudio::previousSubsong() {
	if(!anyOpened) return;
	if (currentSong>0) currentSong--; else currentSong=maximumSong-1;
	sapPlaySong(currentSong);
	
}

void SAPAudio::updateSongChr() {
	if (anyOpened) {
		sprintf(songChr,"%d/%d",currentSong+1,maximumSong);
	}
	
}

int SAPAudio::getPaused() {
	return paused;
}

 
